--EnhancedChatFilter 用户设置
local chatLinesLimit		--多少行内不许重复
local stringDifferenceLimit	--重复信息判定值, 0..1, 0为完全相同

--EnhancedChatFilter 内部定义
local ecfVersioin = GetAddOnMetadata("EnhancedChatFilter","Version")
local _, ecf = ...
local utf8replace = ecf.utf8replace -- utf8.lua
local L = ecf.L -- locales.lua
local debugMode = nil
local chatLines = {}
local prevLineID = 0
local filterResult = nil
local filterCharList = "[|@!/<>\"`'_#&;:~\\]"
local filterCharListRegex = "[%(%)%.%%%+%-%*%?%[%]%$%^={}]"

local scrollLineLimit = 12

local ecfFrame = CreateFrame("Frame")
local login = nil
local allowWisper = {}

local gsub, select, ipairs, tremove, tinsert, pairs, strsub, getn, sort, format, tonumber, strfind, strlen, strmatch =
gsub, select, ipairs, tremove, tinsert, pairs, strsub, getn, sort, format, tonumber, string.find, strlen, strmatch

--信息中将被过滤器过滤的 UTF-8 符号（常被用于干扰）
local UTF8Symbols = {['·']='',['＠']='',['＃']='',['％']='',
	['＆']='',['＊']='',['——']='',['＋']='',['｜']='',['～']='',['　']='',
	['，']='',['。']='',['、']='',['？']='',['！']='',['：']='',['；']='',
	['’']='',['‘']='',['“']='',['”']='',['【']='',['】']='',['『']='',
	['』']='',['《']='',['》']='',['（']='',['）']='',['￥']='',['＝']='',
	['…']='',['……']='',['１']='1',['２']='2',['３']='3',['４']='4',['５']='5',
	['６']='6',['７']='7',['８']='8',['９']='9',['０']='0',['⒈']='1',['⒉']='2',
	['⒊']='3',['⒋']='4',['⒌']='5',['⒍']='6',['⒎']='7',['⒏']='8',['⒐']='9',
	['〔']='',['〕']='',['〈']='',['〉']='',['‖']=''}

--http://www.wowwiki.com/USERAPI_StringHash
local function StringHash(text)
	local counter = 1
	local len = string.len(text)
	for i = 1, len, 3 do
	counter = math.fmod(counter*8161, 4294967279) +  -- 2^32 - 17: Prime!
		(string.byte(text,i)*16776193) +
		((string.byte(text,i+1) or (len-i+256))*8372226) +
		((string.byte(text,i+2) or (len-i+256))*3932164)
	end
	return math.fmod(counter, 4294967291) -- 2^32 - 5: Prime (and different from the prime in the loop)
end

local EnhancedChatFilter = LibStub("AceAddon-3.0"):NewAddon("EnhancedChatFilter", "AceConsole-3.0", "AceEvent-3.0")

--数据库初始设置
local defaults = {
	profile = {
		--ECF聊天过滤总开关
		enableFilter = true,
		--启用密语白名单过滤
		enableWisper = false,
		--启用“忙碌”玩家过滤
		enableDND = true,
		--启用重复聊天过滤
		enableRPT = true,
		--启用成就刷屏过滤
		enableCFA = true,
		--启用RaidAlert刷屏过滤
		enableRAF = false,
		--启用QuestReport刷屏过滤
		enableQRF = false,
		--启用扩展屏蔽名单
		enableIGM = false,
		--黑名单列表
		blackList = {},
		--扩展屏蔽名单
		ignoreMoreList = {},
		--小地图图标选项
		minimap = {
			hide = false,
			enableMinimapIcon = true,
		},
		chatLinesLimit = 20,
		stringDifferenceLimit = 0.1
	}
}

--创建小地图图标数据
local ecfLDB = LibStub("LibDataBroker-1.1"):NewDataObject("Enhanced Chat Filter", {
	type = "data source",
	text = "Enhanced Chat Filter",
	icon = "Interface\\Icons\\Trade_Archaeology_Orc_BloodText",
	OnClick = function() EnhancedChatFilter:EnhancedChatFilterOpen() end,
	OnTooltipShow = function(tooltip)
		tooltip:AddLine("|cffecf0f1Enhanced Chat Filter|r\n"..L["ClickToOpenConfig"])
	end
})

--创建 Libstub 变量
local icon = LibStub("LibDBIcon-1.0")

--初始化插件
function EnhancedChatFilter:OnInitialize()
	EnhancedChatFilter:RegisterChatCommand("ecf", "EnhancedChatFilterOpen")
	EnhancedChatFilter:RegisterChatCommand("ecf-clear", "EnhancedChatFilterClear")
	EnhancedChatFilter:RegisterChatCommand("ecf-iml", "EnhancedChatFilterIgnoreMoreList")
	EnhancedChatFilter:RegisterChatCommand("ecf-cleariml", "EnhancedChatFilterClearIMList")
	EnhancedChatFilter:RegisterChatCommand("ecf-debug", "EnhancedChatFilterDebug")

	self.db = LibStub("AceDB-3.0"):New("ecfDB", defaults, "Default")
	icon:Register("Enhanced Chat Filter", ecfLDB, self.db.profile.minimap)
	chatLinesLimit = EnhancedChatFilter:GetchatLinesLimit()
	stringDifferenceLimit = EnhancedChatFilter:GetstringDifferenceLimit()
end

--清空关键词确认对话框
StaticPopupDialogs["ECF_CLEAR_BLACKLIST"] = {
	text = L["DoYouWantToClearBlackList?"],
	button1 = L["Yes"],
	button2 = L["No"],
	OnAccept = function() EnhancedChatFilter:EnhancedChatFilterClear() end,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3, -- avoid some UI taint
}

--获取/设置数据库数据的函数
function EnhancedChatFilter:GetEnableFilter()
	return self.db.profile.enableFilter
end

function EnhancedChatFilter:ToggleEnableFilter(value)
	self.db.profile.enableFilter = value
end

function EnhancedChatFilter:GetEnableWisper()
	return self.db.profile.enableWisper
end

function EnhancedChatFilter:ToggleEnableWisper(value)
	self.db.profile.enableWisper = value
end

function EnhancedChatFilter:GetEnableDND()
	return self.db.profile.enableDND
end

function EnhancedChatFilter:ToggleEnableDND(value)
	self.db.profile.enableDND = value
end

function EnhancedChatFilter:GetEnableRPT()
	return self.db.profile.enableRPT
end

function EnhancedChatFilter:ToggleEnableRPT(value)
	self.db.profile.enableRPT = value
end

function EnhancedChatFilter:GetEnableCFA()
	return self.db.profile.enableCFA
end

function EnhancedChatFilter:ToggleEnableCFA(value)
	self.db.profile.enableCFA = value
end

function EnhancedChatFilter:GetEnableRAF()
	return self.db.profile.enableRAF
end

function EnhancedChatFilter:ToggleEnableRAF(value)
	self.db.profile.enableRAF = value
end

function EnhancedChatFilter:GetEnableQRF()
	return self.db.profile.enableQRF
end

function EnhancedChatFilter:ToggleEnableQRF(value)
	self.db.profile.enableQRF = value
end

function EnhancedChatFilter:GetEnableIGM()
	return self.db.profile.enableIGM
end

function EnhancedChatFilter:ToggleEnableIGM(value)
	self.db.profile.enableIGM = value
end

function EnhancedChatFilter:GetEnableMinimapIcon()
	return self.db.profile.minimap.enableMinimapIcon
end

function EnhancedChatFilter:ToggleEnableMinimapIcon(value)
	self.db.profile.minimap.enableMinimapIcon = value
end

function EnhancedChatFilter:GetMinimapStatus()
	return self.db.profile.minimap.hide
end

function EnhancedChatFilter:SetMinimapStatus(value)
	self.db.profile.minimap.hide = value
end

function EnhancedChatFilter:GetBlackList()
	return self.db.profile.blackList
end

function EnhancedChatFilter:InsertBlackList(value, typeModus)
	table.insert(self.db.profile.blackList,{value, typeModus})
end

function EnhancedChatFilter:RemoveFromBlackList(index)
	table.remove(self.db.profile.blackList,index)
end

function EnhancedChatFilter:ClearBlackList()
	self.db.profile.blackList = {}
end

function EnhancedChatFilter:GetIgnoreMoreList()
	return self.db.profile.ignoreMoreList
end

function EnhancedChatFilter:InsertIgnoreMoreList(value)
	table.insert(self.db.profile.ignoreMoreList,{value})
end

function EnhancedChatFilter:RemoveFromIgnoreMoreList(index)
	table.remove(self.db.profile.ignoreMoreList,index)
end

function EnhancedChatFilter:ClearIgnoreMoreList()
	self.db.profile.ignoreMoreList = {}
end

function EnhancedChatFilter:GetchatLinesLimit()
	return self.db.profile.chatLinesLimit
end

function EnhancedChatFilter:SetchatLinesLimit(value)
	self.db.profile.chatLinesLimit = value
end

function EnhancedChatFilter:GetstringDifferenceLimit()
	return self.db.profile.stringDifferenceLimit
end

function EnhancedChatFilter:SetstringDifferenceLimit(value)
	self.db.profile.stringDifferenceLimit = value
end

--扩展黑名单
local function SendMessage(event, msg)
	local info = ChatTypeInfo[strsub(event, 10)]
	for i = 1, NUM_CHAT_WINDOWS do
		local ChatFrames = _G["ChatFrame"..i]
		if (ChatFrames and ChatFrames:IsEventRegistered(event)) then
			ChatFrames:AddMessage(msg, info.r, info.g, info.b)
		end
	end
end

local function ignoreMore(player)
	if (EnhancedChatFilter:GetEnableIGM() ~= true) then return end
	if (not player) then return end
	local ignore = nil
	if GetNumIgnores() >= 50 then
		for i = 1, GetNumIgnores() do
			local name = GetIgnoreName(i)
			if (player == name) then
				ignore = true
				break
			end
		end
		if (not ignore) then
			local trimmedPlayer = Ambiguate(player, "none")
			EnhancedChatFilter:InsertIgnoreMoreList(trimmedPlayer)
			if debugMode then print("Added to ECF ignoreMoreList!") end
			SendMessage("CHAT_MSG_SYSTEM", format(ERR_IGNORE_ADDED_S, trimmedPlayer))
		end
	end
end

hooksecurefunc("AddIgnore", ignoreMore)
hooksecurefunc("AddOrDelIgnore", ignoreMore)

--禁用暴雪内部语言过滤器
local GetCVar,SetCVar = GetCVar,SetCVar

local profanityFilter=CreateFrame("Frame")
profanityFilter:Hide()

local function allowSwearing()
	if GetCVar("profanityFilter")~="0" then
		SetCVar("profanityFilter", "0")
	end
end

profanityFilter:SetScript("OnEvent", allowSwearing)
profanityFilter:RegisterEvent("VARIABLES_LOADED")
profanityFilter:RegisterEvent("CVAR_UPDATE")
profanityFilter:RegisterEvent("PLAYER_ENTERING_WORLD")
profanityFilter:RegisterEvent("BN_MATURE_LANGUAGE_FILTER")
profanityFilter:RegisterEvent("BN_CONNECTED")

allowSwearing() -- 确认再次载入

--登陆/好友列表更新时同步更新允许密语玩家列表
ecfFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
ecfFrame:RegisterEvent("FRIENDLIST_UPDATE")
ecfFrame:SetScript("OnEvent", function(self, event)
	if event == "PLAYER_ENTERING_WORLD" then
		ShowFriends() --强制刷新一次好友列表
	else
		if not login then --登陆时只添加一次
			login = true
			local num = GetNumFriends()
			for i = 1, num do
				local n = GetFriendInfo(i)
				--添加好友到允许名单
				if n then allowWisper[n] = true end
			end
			return
		end
	end
	if debugMode then for k in pairs(allowWisper) do print("allowed: "..k) end end
end)

--自己主动密语的玩家加入安全名单
local function addToAllowWisper(self,_,_,player)
	local trimmedPlayer = Ambiguate(player, "none")
	if allowWisper[trimmedPlayer] then return end --Do nothing if on safe list
	allowWisper[trimmedPlayer] = true --If we want to whisper someone, they're good
end

--字符串不同程度, 范围0..1, 0为完全相同, 1为完全不同
local function stringDifference(stringA, stringB)
	local len_a, len_b = #stringA, #stringB
	local templast, temp = {}, {}
	for j=0, len_b do templast[j+1] = j end
	for i=1, len_a do
		temp[1] = i
		for j=1, len_b do
			if(stringA:sub(i,i) == stringB:sub(j,j)) then
				temp[j+1] = templast[j]
			else
				temp[j+1] = min(templast[j+1], temp[j], templast[j]) + 1
			end
		end
		for j=0, len_b do templast[j+1]=temp[j+1] end
	end
	return temp[len_b+1]/(len_a+len_b)
end

-------------------------------------- Filters ------------------------------------

local chatChannel = {["CHAT_MSG_WHISPER"] = 1, ["CHAT_MSG_SAY"] = 2, ["CHAT_MSG_CHANNEL"] = 3, ["CHAT_MSG_YELL"] = 3, ["CHAT_MSG_PARTY"] = 4, ["CHAT_MSG_PARTY_LEADER"] = 4, ["CHAT_MSG_RAID"] = 4, ["CHAT_MSG_RAID_LEADER"] = 4, ["CHAT_MSG_RAID_WARNING"] = 4, ["CHAT_MSG_INSTANCE_CHAT"] = 4,["CHAT_MSG_INSTANCE_CHAT_LEADER"] = 4, ["CHAT_MSG_DND"] = 101}

local function ECFfilter(self,event,msg,player,_,_,_,flags,_,_,_,_,lineID)
	--如果不启用过滤直接返回
	if(EnhancedChatFilter:GetEnableFilter() ~= true) then return end

	--避免自己被过滤
	local trimmedPlayer = Ambiguate(player, "none")
	if UnitIsUnit(trimmedPlayer,"player") then return end

	--对 GM 和 DEV 的密语不做处理
	if type(flags) == "string" and (flags == "GM" or flags == "DEV") then return end

	--如果这行被处理过，则直接引用之前的结果
	if(lineID == prevLineID) then
		return filterResult
	else
		prevLineID = lineID
		filterResult = nil
	end

	if(EnhancedChatFilter:GetEnableIGM() == true and chatChannel[event] <= 1) then --拓展黑名单过滤
		local ignoreMoreList = EnhancedChatFilter:GetIgnoreMoreList()
		for _,ignorePlayer in ipairs(ignoreMoreList) do
			if (trimmedPlayer == ignorePlayer[1]) then
				if debugMode then print(trimmedPlayer.." Muted~!") end
				filterResult = true
				return true
			end
		end
	end

	if(EnhancedChatFilter:GetEnableWisper() == true and chatChannel[event] <= 1) then --密语白名单过滤
		--强制刷新一次好友列表
		ShowFriends()
		--同工会，同团队，同小队和好友不做处理
		if allowWisper[trimmedPlayer] or UnitIsInMyGuild(trimmedPlayer) or UnitInRaid(trimmedPlayer) or UnitInParty(trimmedPlayer) then return end
		--战网好友不做处理
		for i = 1, select(2, BNGetNumFriends()) do
			local GameAccount = BNGetNumFriendGameAccounts(i)
			for j = 1, GameAccount do
				local _, rName, rGame = BNGetFriendGameAccountInfo(i, j)
				if (rName == trimmedPlayer and rGame == "WoW") then return end
			end
		end
		if debugMode then print(player.."'s wisper blocked.") end
		filterResult = true
		return true
	end

	if(EnhancedChatFilter:GetEnableDND() == true and (chatChannel[event] <= 3 or chatChannel[event] == 101)) then --"忙碌"玩家过滤
		if type(flags) == "string" and flags == "DND" then
			filterResult = true
			return true
		end
	end

	if(chatChannel[event] <= 3) then --关键词过滤

		--从数据库读取黑名单
		local blacklist = EnhancedChatFilter:GetBlackList()
		--if debugMode then print("RAWMsg: "..msg) end

		--删除颜色代码, 物品链接标志, 空格以及一些符号
		local filterString = msg:upper():gsub("|C[0-9A-F]+",""):gsub("|H[^|]+|H",""):gsub("|H|R",""):gsub("%s", ""):gsub(filterCharList, "")

		--删除 UTF-8 干扰符
		filterString = utf8replace(filterString, UTF8Symbols)
		local newfilterString = filterString:gsub(filterCharListRegex, "")

		if debugMode then print("filterString: "..filterString) end

		--从处理过的聊天信息中过滤包含黑名单词语的聊天内容
		for _, blacklistWord in ipairs(blacklist) do
			local keyWord = blacklistWord[1]
			local currentString
			if (blacklistWord[2] == nil) then -- 如果不是正则，也过滤正则中使用的特殊字符
				keyWord = keyWord:upper()
				currentString = newfilterString
			else
				currentString = filterString
			end
			--检查常规黑名单
			if (strfind(currentString,keyWord)) then
				if debugMode then print("Trigger: "..keyWord) end
				filterResult = true
				return true --deletion of message from chat
			end
		end
	end

	local RaidAlertTagList = {"%*%*(.+)%*%*", "EUI:(.+)施放了", "EUI:(.+)中断", "EUI:(.+)就绪"}  --RaidAlert 特征
	local QuestReportTagList = {"任务进度提示%s?[:：]", "%(任务完成%)", "<大脚组队提示>", "%[接受任务%]", "<大脚团队提示>"} --QuestReport 特征

	if (EnhancedChatFilter:GetEnableRAF() == true and IsInGroup() and (chatChannel[event] == 4 or chatChannel[event] == 2)) then
		for _,RaidAlertTag in ipairs(RaidAlertTagList) do
			if(msg:find(RaidAlertTag)) then
				if debugMode then print("Trigger: "..RaidAlertTag.."in "..msg) end
				filterResult = true
				return true
			end
		end
	end

	if (EnhancedChatFilter:GetEnableQRF() == true and IsInGroup() and (chatChannel[event] == 4 or chatChannel[event] == 2)) then
		for _,QuestReportTag in ipairs(QuestReportTagList) do
			if(msg:find(QuestReportTag)) then
				if debugMode then print("Trigger: "..QuestReportTag.."in "..msg) end
				filterResult = true
				return true
			end
		end
	end

	if(EnhancedChatFilter:GetEnableRPT() == true and chatChannel[event] <= 4) then --重复信息过滤
		--删除掉表情和所有标点符号, 但如果对话只有这些内容则保留
		local msgLine = msg:gsub("%p+", ""):gsub("{.*}","")
		if(msgLine == "") then msgLine = msg end

		--处理重复信息
		local msgtable = {Sender = trimmedPlayer, Msg = msgLine, Time = GetTime()}
		for i=1, #chatLines do
			--如果一个人在0.6秒钟之内发了多条信息, 过滤
			--如果一个人的信息完全相同, 过滤
			if (chatLines[i].Sender == msgtable.Sender and ((msgtable.Time - chatLines[i].Time) < 0.600 or stringDifference(chatLines[i].Msg,msgtable.Msg) <= stringDifferenceLimit)) then
				chatLines[i] = msgtable
				filterResult = true
				return true
			end
			if i == chatLinesLimit then tremove(chatLines, 1) end
		end
		tinsert(chatLines, msgtable)
	end
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", addToAllowWisper)
ChatFrame_AddMessageEventFilter("CHAT_MSG_DND", ECFfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", ECFfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", ECFfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", ECFfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", ECFfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", ECFfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY_LEADER", ECFfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID", ECFfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_LEADER", ECFfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_WARNING", ECFfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_INSTANCE_CHAT", ECFfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_INSTANCE_CHAT_LEADER", ECFfilter)

---------------------------------------FRAMES---------------------------------------------------------

--main frames
local blackListFrame = CreateFrame("Frame","ConfigFrame",UIParent,"BasicFrameTemplate")
local blackListScrollFrame = CreateFrame("Frame", "ScrollFrame", blackListFrame)
local toggleFrame = CreateFrame("Frame", "ToggleFrame", blackListFrame)
local stringConfigFrame = CreateFrame("Frame", "stringConfigFrame", blackListFrame)
local advancedConfigFrame = CreateFrame("Frame", "advancedConfigFrame", blackListFrame)

--toggles, input fields, buttons etc
local blackListField = CreateFrame("EditBox", "blackListField", blackListScrollFrame,"InputBoxTemplate")
local hashStringIOField = CreateFrame("EditBox", "hashStringIOField", stringConfigFrame,"InputBoxTemplate")

local MainToggle = CreateFrame("CheckButton", "MainToggle", toggleFrame,"UICheckButtonTemplate")
local MinimapToggle = CreateFrame("CheckButton", "MinimapToggle", toggleFrame,"UICheckButtonTemplate")
local WisperWhiteToggle = CreateFrame("CheckButton", "WisperWhiteToggle", toggleFrame,"UICheckButtonTemplate")
local DNDFilterToggle = CreateFrame("CheckButton", "DNDFilterToggle", toggleFrame,"UICheckButtonTemplate")
local RPTToggle = CreateFrame("CheckButton", "RPTToggle", toggleFrame,"UICheckButtonTemplate")
local CFAFilterToggle = CreateFrame("CheckButton", "CFAFilterToggle", toggleFrame,"UICheckButtonTemplate")
local RAFToggle = CreateFrame("CheckButton", "RAFToggle", toggleFrame,"UICheckButtonTemplate")
local QRFToggle = CreateFrame("CheckButton", "QRFToggle", toggleFrame,"UICheckButtonTemplate")
local IGMToggle = CreateFrame("CheckButton", "IGMToggle", toggleFrame,"UICheckButtonTemplate")
local EnableRegexToggle = CreateFrame("CheckButton", "EnableRegexToggle", blackListScrollFrame,"UICheckButtonTemplate")

local removeButton = CreateFrame("Button", "removeButton", blackListScrollFrame, "UIPanelButtonTemplate")
local addButton = CreateFrame("Button", "addButton", blackListScrollFrame, "UIPanelButtonTemplate")
local arrangeButton = CreateFrame("Button", "arrangeButton", blackListScrollFrame, "UIPanelButtonTemplate")
local clearUpButton = CreateFrame("Button", "clearUpButton", blackListScrollFrame, "UIPanelButtonTemplate")

local importButton = CreateFrame("Button", "importButton", stringConfigFrame, "UIPanelButtonTemplate")
local exportButton = CreateFrame("Button", "exportButton", stringConfigFrame, "UIPanelButtonTemplate")

local chatLinesLimitSlider = CreateFrame("Slider", "chatLinesLimitSlider", advancedConfigFrame, "OptionsSliderTemplate")
local stringDifferenceLimitSlider = CreateFrame("Slider", "stringDifferenceLimitSlider", advancedConfigFrame, "OptionsSliderTemplate")

--text
local titleText = blackListFrame:CreateFontString("frametext","ARTWORK","GameFontNormal")
local titleAddText = blackListField:CreateFontString("frametext","ARTWORK","GameFontNormal")
local toggleText = MainToggle:CreateFontString("frametext","ARTWORK","GameFontNormal")
local minimapToggleText = MinimapToggle:CreateFontString("frametext","ARTWORK","GameFontNormal")
local wisperToggleText = WisperWhiteToggle:CreateFontString("frametext","ARTWORK","GameFontNormal")
local dndToggleText = DNDFilterToggle:CreateFontString("frametext","ARTWORK","GameFontNormal")
local rptToggleText = RPTToggle:CreateFontString("frametext","ARTWORK","GameFontNormal")
local cfaToggleText = CFAFilterToggle:CreateFontString("frametext","ARTWORK","GameFontNormal")
local rafToggleText = RAFToggle:CreateFontString("frametext","ARTWORK","GameFontNormal")
local qrfToggleText = QRFToggle:CreateFontString("frametext","ARTWORK","GameFontNormal")
local igmToggleText = IGMToggle:CreateFontString("frametext","ARTWORK","GameFontNormal")
local regexToggleText = EnableRegexToggle:CreateFontString("frametext","ARTWORK","GameFontNormal")
local chatLinesLimitValue = chatLinesLimitSlider:CreateFontString("frametext","ARTWORK","GameFontNormal")
local stringDifferenceLimitValue = stringDifferenceLimitSlider:CreateFontString("frametext","ARTWORK","GameFontNormal")

--tabs

local tabsFrame = CreateFrame("Frame","tabsFrame",blackListFrame)

local tabList={}

local function tabListAdded(line,frame) -- frame is the config frame that you want to show
	tabList[line].showframe[frame]=true
end

local function tabFrameShowHide(line)
	for i=1, 4 do
		for frame in pairs(tabList[i].showframe) do
			if(i ~= line) then frame:Hide() else frame:Show() end
		end
	end
end

local function tabInitialize()
	for i=1, 4 do
		tabList[i]=CreateFrame("Button",nil,tabsFrame,"UIPanelButtonTemplate")
		tabList[i]:SetSize(170,30)
		tabList[i]:SetPoint("TOPLEFT",tabsFrame,"TOPLEFT",5,(i-1)*-30-5)
		tabList[i].showframe={}
		tabList[i]:SetScript("OnClick",function() tabFrameShowHide(i) end)
	end

	tabListAdded(1,toggleFrame)
	tabList[1]:SetText(L["General"])

	tabListAdded(2,blackListScrollFrame)
	tabList[2]:SetText(L["BlackworldListSetting"])

	tabListAdded(3,stringConfigFrame)
	tabList[3]:SetText(L["StringIO"])

	tabListAdded(4,advancedConfigFrame)
	tabList[4]:SetText(L["AdvancedConfigFrame"])

	-- Show Tab 1
	tabFrameShowHide(1)
end

--
local scrollHighlight = 0
local scrollOffset = 0

--
local function tablelength(T)
	local count = 0
	for _ in pairs(T) do count = count + 1 end
	return count
end

local function SetToggleText(check, typeToggle)
	local filterType={
		["filter"] = {toggleText, MainToggle, L["MainFilter"].." |cff2ed689"..L["Enabled"], L["MainFilter"].." |cffE2252D"..L["Disabled"]},
		["minimap"] = {minimapToggleText, MinimapToggle, L["MinimapButton"].." |cff2ed689"..L["Enabled"], L["MinimapButton"].." |cffE2252D"..L["Disabled"]},
		["wisper"] = {wisperToggleText, WisperWhiteToggle, L["WhisperWhitelistMode"].." |cff2ed689"..L["Enabled"].."|cffE2252D"..L["BeCareful"], L["WhisperWhitelistMode"].." |cffE2252D"..L["Disabled"]},
		["dnd"] = {dndToggleText, DNDFilterToggle, L["DNDfilter"].." |cff2ed689"..L["Enabled"], L["DNDfilter"].." |cffE2252D"..L["Disabled"]},
		["rpt"] = {rptToggleText, RPTToggle, L["RepeatFilter"].." |cff2ed689"..L["Enabled"], L["RepeatFilter"].." |cffE2252D"..L["Disabled"]},
		["cfa"] = {cfaToggleText, CFAFilterToggle, L["AchievementFilter"].." |cff2ed689"..L["Enabled"], L["AchievementFilter"].." |cffE2252D"..L["Disabled"]},
		["raf"] = {rafToggleText, RAFToggle, L["RaidAlertFilter"].." |cff2ed689"..L["Enabled"], L["RaidAlertFilter"].." |cffE2252D"..L["Disabled"]},
		["qrf"] = {qrfToggleText, QRFToggle, L["QuestReportFilter"].." |cff2ed689"..L["Enabled"], L["QuestReportFilter"].." |cffE2252D"..L["Disabled"]},
		["igm"] = {igmToggleText, IGMToggle, L["IgnoreMoreList"].." |cff2ed689"..L["Enabled"], L["IgnoreMoreList"].." |cffE2252D"..L["Disabled"]}
	}
	filterType[typeToggle][1]:SetText(filterType[typeToggle][(check and 3) or 4])
	filterType[typeToggle][2]:SetChecked(check)
end

local function checkBlacklist(blackWord, typeModus) -- 确保关键词自身不会被预处理干扰
	local newWord = blackWord:gsub("%s", ""):gsub(filterCharList, "")
	if (typeModus == nil) then newWord=newWord:gsub(filterCharListRegex, "") end
	newWord = utf8replace(newWord, UTF8Symbols)
	if(newWord ~= blackWord) then return true end
end

local function unlockHighlight()
	for i=1,scrollLineLimit do
		blackListScrollFrame.list[i]:UnlockHighlight()
	end
end

-- Tooltips
local function SetToolTip(frame, text)
	if text then
		frame.tooltipText = text
		frame:SetScript("OnEnter", function(self)
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
			GameTooltip:SetText(self.tooltipText, nil, nil, nil, nil, true)
			GameTooltip:Show()
		end)
		frame:SetScript("OnLeave", function() GameTooltip:Hide() end)
	else
		frame:SetScript("OnEnter", nil)
		frame:SetScript("OnLeave", nil)
	end
end

local function BuildUpFrame()
	blackListFrame:SetSize(700,300) -- 主框体
	blackListFrame:SetPoint("Center",UIParent,"CENTER",0,0)
	blackListFrame:SetFrameStrata("HIGH")
	blackListFrame:SetBackdrop({bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", tile = true, tileSize = 16, edgeSize = 16,insets = { left = 4, right = 4, top = 4, bottom = 4 }})
-- 主框体可以移动 --
	blackListFrame:SetMovable(true)
	blackListFrame:EnableMouse(true)
	blackListFrame:RegisterForDrag("LeftButton")
	blackListFrame:SetScript("OnDragStart", blackListFrame.StartMoving)
	blackListFrame:SetScript("OnDragStop", blackListFrame.StopMovingOrSizing)
-- 鼠标离开插件区域时界面变为半透明 --
	blackListFrame:SetScript("OnUpdate", function(self)
		local current = GetMouseFocus()
		-- Scan through the parent chain
		while current ~= nil do
			if current == self then
				self:SetAlpha(1.0) -- active
				return
			end
			current = current:GetParent()
		end
		self:SetAlpha(0.5) -- inactive
	end)
	blackListFrame:Hide()
-- 主框体可以被Escape关闭 --
	tinsert(UISpecialFrames, blackListFrame:GetName())

	tabsFrame:SetPoint("LEFT",blackListFrame,"LEFT",2,-10)
	tabsFrame:SetSize(180,270)

	blackListScrollFrame:SetPoint("LEFT",tabsFrame,"RIGHT",0,0)
	blackListScrollFrame:SetSize(250,10+scrollLineLimit*20) -- 关键词列表
	blackListScrollFrame:SetBackdrop({edgeFile = "Interface/Tooltips/UI-Tooltip-Border",tile = true, tileSize = 16, edgeSize = 16,insets = { left = 4, right = 4, top = 4, bottom = 4 }})

	MainToggle:SetPoint("TOPLEFT",tabsFrame,"TOPRIGHT",30,-5)
	MainToggle:SetScript("OnClick", function(self)
		EnhancedChatFilter:GetEnableFilter()
		local check = self:GetChecked()
		EnhancedChatFilter:ToggleEnableFilter(check)
		SetToggleText(check, 'filter')
	end)

	MinimapToggle:SetPoint("TOPLEFT",MainToggle,"BOTTOMLEFT",0,7)
	MinimapToggle:SetScript("OnClick", function(self)
		local check = self:GetChecked()
		EnhancedChatFilter:ToggleEnableMinimapIcon(check)
		SetToggleText(check, 'minimap')
		EnhancedChatFilter:SetMinimapStatus(check)
		if(check) then icon:Show("Enhanced Chat Filter") else icon:Hide("Enhanced Chat Filter") end
	end)

	WisperWhiteToggle:SetPoint("TOPLEFT",MinimapToggle,"BOTTOMLEFT",0,7)
	SetToolTip(WisperWhiteToggle,L["WhisperWhitelistModeTooltip"])
	WisperWhiteToggle:SetScript("OnClick", function(self)
		local check = self:GetChecked()
		EnhancedChatFilter:ToggleEnableWisper(check)
		SetToggleText(check, 'wisper')
	end)

	DNDFilterToggle:SetPoint("TOPLEFT",WisperWhiteToggle,"BOTTOMLEFT",0,7)
	SetToolTip(DNDFilterToggle,L["DNDfilterTooltip"])
	DNDFilterToggle:SetScript("OnClick", function(self)
		local check = self:GetChecked()
		EnhancedChatFilter:ToggleEnableDND(check)
		SetToggleText(check, 'dnd')
	end)

	RPTToggle:SetPoint("TOPLEFT",DNDFilterToggle,"BOTTOMLEFT",0,7)
	SetToolTip(RPTToggle,L["RepeatFilterTooltip"])
	RPTToggle:SetScript("OnClick", function(self)
		local check = self:GetChecked()
		EnhancedChatFilter:ToggleEnableRPT(check)
		SetToggleText(check, 'rpt')
	end)

	CFAFilterToggle:SetPoint("TOPLEFT",RPTToggle,"BOTTOMLEFT",0,7)
	SetToolTip(CFAFilterToggle,L["AchievementFilterTooltip"])
	CFAFilterToggle:SetScript("OnClick", function(self)
		local check = self:GetChecked()
		EnhancedChatFilter:ToggleEnableCFA(check)
		SetToggleText(check, 'cfa')
	end)

	RAFToggle:SetPoint("TOPLEFT",CFAFilterToggle,"BOTTOMLEFT",0,7)
	SetToolTip(RAFToggle,L["RaidAlertFilterTooltip"])
	RAFToggle:SetScript("OnClick", function(self)
		local check = self:GetChecked()
		EnhancedChatFilter:ToggleEnableRAF(check)
		SetToggleText(check, 'raf')
	end)

	QRFToggle:SetPoint("TOPLEFT",RAFToggle,"BOTTOMLEFT",0,7)
	SetToolTip(QRFToggle,L["QuestReportFilterTooltip"])
	QRFToggle:SetScript("OnClick", function(self)
		local check = self:GetChecked()
		EnhancedChatFilter:ToggleEnableQRF(check)
		SetToggleText(check, 'qrf')
	end)

	IGMToggle:SetPoint("TOPLEFT",QRFToggle,"BOTTOMLEFT",0,7)
	IGMToggle:SetScript("OnClick", function(self)
		local check = self:GetChecked()
		EnhancedChatFilter:ToggleEnableIGM(check)
		SetToggleText(check, 'igm')
	end)

	EnableRegexToggle:SetPoint("LEFT",blackListField,"RIGHT",0,-2)
	SetToolTip(EnableRegexToggle,L["RegexTooltip"])

	blackListField:SetPoint("TOPLEFT",ScrollFrame,"TOPRIGHT",20,-20) -- 关键词输入区
	blackListField:SetSize(180,20)
	blackListField:SetMaxLetters(40)
	blackListField:SetTextInsets(5, 5, 2, 2)
	blackListField:SetText("")
	blackListField:SetAutoFocus(false)
	blackListField:SetScript("OnTextChanged", function(self,userinput)
		if(userinput == true) then
			if(self:GetText() ~= "") then
				addButton:Enable()
			else
				addButton:Disable()
			end
		end
	end)

	hashStringIOField:SetPoint("TOPLEFT",tabsFrame,"TOPRIGHT",30,-30) -- 字符串导入导出区
	hashStringIOField:SetSize(300,20)
	hashStringIOField:SetTextInsets(5, 5, 2, 2)
	hashStringIOField:SetText("")
	hashStringIOField:SetAutoFocus(false)
	hashStringIOField:SetScript("OnTextChanged", function(self,userinput)
		if(userinput == true) then
			local IOnewWord = self:GetText()
			if(IOnewWord ~= "") then
				importButton:Enable()
			else
				importButton:Disable()
			end
		end
	end)

	importButton:SetSize(145,30) -- 导入按钮
	importButton:SetText(L["Import"])
	importButton:SetPoint("TOPLEFT",hashStringIOField,"BOTTOMLEFT",0,-5)
	importButton:SetScript("OnClick", function()
		local importString = hashStringIOField:GetText()
		local wordString, HashString = strsplit("@", importString)
		if (tonumber(HashString) ~= tonumber(StringHash(wordString))) then
			print(L["StringHashMismatch"])
			hashStringIOField:SetFocus()
			hashStringIOField:HighlightText()
			return
		end

		local newBlackList = {strsplit(";", wordString)}
		for _, blacklist in ipairs(newBlackList) do
			if (blacklist ~= nil) then
				local imNewWord, imTypeWord = strsplit(",",blacklist)
				if (checkBlacklist(imNewWord, imTypeWord) == true) then
					print(imNewWord..L["IncludeAutofilteredWord"])
				else
					EnhancedChatFilter:InsertBlackList(imNewWord,imTypeWord)
				end
			end
		end
		blackListScrollFrame.ScrollFrameUpdate()
		hashStringIOField:SetText("")
		hashStringIOField:ClearFocus()
		importButton:Disable()
		print(L["ImportSucceeded"])
	end)
	importButton:Disable()

	exportButton:SetSize(145,30) -- 导出按钮
	exportButton:SetText(L["Export"])
	exportButton:SetPoint("LEFT",importButton,"RIGHT",2,0)
	exportButton:SetScript("OnClick", function()
		local blacklist = EnhancedChatFilter:GetBlackList()
		local blackStringList = {}
		for _, blackWord in ipairs(blacklist) do
			if (checkBlacklist(blackWord[1], blackWord[2]) == true) then
				print(blackWord[1]..L["IncludeAutofilteredWord"])
			else
				if (blackWord[2] ~= nil) then
					blackStringList[#blackStringList+1] = blackWord[1]..","..blackWord[2]
				else
					blackStringList[#blackStringList+1] = blackWord[1]
				end
			end
		end
		local blackString = table.concat(blackStringList,";")
		local hashString = StringHash(blackString)
		hashStringIOField:SetText(blackString.."@"..hashString)
		hashStringIOField:SetFocus()
		hashStringIOField:HighlightText()
	end)

	addButton:SetSize(205,30) -- 添加关键词按钮
	addButton:SetText(L["Add"])
	addButton:SetPoint("TOPLEFT",blackListField,"BOTTOMLEFT",-2,-4)
	addButton:SetScript("OnClick", function()
		local typeModus = EnableRegexToggle:GetChecked()
		if (typeModus == true) then typeModus = 'regex' else typeModus = nil end
		local newWord = blackListField:GetText()
		if (checkBlacklist(newWord, typeModus) == true) then
			print(newWord..L["IncludeAutofilteredWord"])
		else
			EnhancedChatFilter:InsertBlackList(newWord, typeModus)
			blackListScrollFrame.ScrollFrameUpdate()
			blackListField:SetText("")
			blackListField:ClearFocus()
		end
		addButton:Disable()
		EnableRegexToggle:SetChecked(false)
	end)
	addButton:Disable()

	removeButton:SetSize(205,30) -- 移除关键词按钮
	removeButton:SetText(L["Remove"])
	removeButton:SetPoint("TOP",addButton,"BOTTOM",0,-5)
	removeButton:SetScript("OnClick", function()
		unlockHighlight()
		EnhancedChatFilter:RemoveFromBlackList(scrollHighlight)
		scrollHighlight = 0
		blackListScrollFrame.ScrollFrameUpdate()
		removeButton:Disable()
	end)
	removeButton:Disable()

	clearUpButton:SetSize(205,30) -- 清空关键词按钮
	clearUpButton:SetText(L["ClearUp"])
	clearUpButton:SetPoint("TOP",removeButton,"BOTTOM",0,-5)
	clearUpButton:SetScript("OnClick", function() StaticPopup_Show("ECF_CLEAR_BLACKLIST") end)

	arrangeButton:SetSize(205,30) -- 整理按钮
	arrangeButton:SetText(L["Arrange"])
	arrangeButton:SetPoint("TOP",clearUpButton,"BOTTOM",0,-5)
	arrangeButton:SetScript("OnClick", function()
		local blacklist = EnhancedChatFilter:GetBlackList()
		local hashWord = {}
		local newBlackList = {}

		for _,v in ipairs(blacklist) do
			if (not hashWord[v[1]]) then
				if(v[2] ~= nil and v[2] ~= "regex") then
					print(v[1]..L["SetAsNormalWord"])
					v[2] = nil
				end
				newBlackList[#newBlackList+1] = {v[1], v[2]}
				hashWord[v[1]] = true
			end
		end

		EnhancedChatFilter:ClearBlackList()

		for _,v in ipairs(newBlackList) do
			if (checkBlacklist(v[1], v[2]) == true) then
				print(v[1]..L["IncludeAutofilteredWord"])
			else
				EnhancedChatFilter:InsertBlackList(v[1], v[2])
			end
		end

		blackListScrollFrame.ScrollFrameUpdate()
	end)

	titleText:SetPoint("CENTER", blackListFrame, "TOP", 0, -10)
	titleText:SetJustifyH("left")
	titleText:SetText("Enhanced Chat Filter MOD Fix v"..ecfVersioin)

	toggleText:SetPoint("LEFT",MainToggle,"RIGHT",5,0)
	toggleText:SetJustifyH("LEFT")
	SetToggleText(EnhancedChatFilter:GetEnableFilter(), 'filter')

	minimapToggleText:SetPoint("LEFT",MinimapToggle,"RIGHT",5,0)
	minimapToggleText:SetJustifyH("LEFT")
	local check = EnhancedChatFilter:GetEnableMinimapIcon()
	SetToggleText(check, 'minimap')
	EnhancedChatFilter:SetMinimapStatus(check)
	if(check) then icon:Show("Enhanced Chat Filter") else icon:Hide("Enhanced Chat Filter") end

	wisperToggleText:SetPoint("LEFT",WisperWhiteToggle,"RIGHT",5,0)
	wisperToggleText:SetJustifyH("LEFT")
	SetToggleText(EnhancedChatFilter:GetEnableWisper(), 'wisper')

	dndToggleText:SetPoint("LEFT",DNDFilterToggle,"RIGHT",5,0)
	dndToggleText:SetJustifyH("LEFT")
	SetToggleText(EnhancedChatFilter:GetEnableDND(), 'dnd')

	rptToggleText:SetPoint("LEFT",RPTToggle,"RIGHT",5,0)
	rptToggleText:SetJustifyH("LEFT")
	SetToggleText(EnhancedChatFilter:GetEnableRPT(), 'rpt')

	cfaToggleText:SetPoint("LEFT",CFAFilterToggle,"RIGHT",5,0)
	cfaToggleText:SetJustifyH("LEFT")
	SetToggleText(EnhancedChatFilter:GetEnableCFA(), 'cfa')

	rafToggleText:SetPoint("LEFT",RAFToggle,"RIGHT",5,0)
	rafToggleText:SetJustifyH("LEFT")
	SetToggleText(EnhancedChatFilter:GetEnableRAF(), 'raf')

	qrfToggleText:SetPoint("LEFT",QRFToggle,"RIGHT",5,0)
	qrfToggleText:SetJustifyH("LEFT")
	SetToggleText(EnhancedChatFilter:GetEnableQRF(), 'qrf')

	igmToggleText:SetPoint("LEFT",IGMToggle,"RIGHT",5,0)
	igmToggleText:SetJustifyH("LEFT")
	SetToggleText(EnhancedChatFilter:GetEnableIGM(), 'igm')

	titleAddText:SetPoint("BOTTOM", blackListField, "TOP", 0,6)
	titleAddText:SetJustifyH("center")
	titleAddText:SetText(L["AddBlackWordTitle"])

	regexToggleText:SetPoint("BOTTOM", EnableRegexToggle, "TOP",0,0)
	regexToggleText:SetJustifyH("center")
	regexToggleText:SetText(L["Regex"].."?")

	blackListScrollFrame.list = {}
	for i=1,scrollLineLimit do
		blackListScrollFrame.list[i] = CreateFrame("Button",nil,blackListScrollFrame,"SecureHandlerClickTemplate")
		blackListScrollFrame.list[i]:SetSize(200,20)
		blackListScrollFrame.list[i]:SetPoint("TOPLEFT",blackListScrollFrame,"TOPLEFT",8,(i-1)*-20-5)
		blackListScrollFrame.list[i]:SetHighlightTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight","ADD") -- yellow
		blackListScrollFrame.list[i]:RegisterForClicks("AnyUp", "AnyDown")

		blackListScrollFrame.list[i].name = blackListScrollFrame.list[i]:CreateFontString(nil,"ARTWORK","GameFontHighlight")
		blackListScrollFrame.list[i].name:SetSize(200-16-8,50)
		blackListScrollFrame.list[i].name:SetJustifyV("CENTER")
		blackListScrollFrame.list[i].name:SetJustifyH("LEFT")

		blackListScrollFrame.list[i]:SetScript("OnClick", function()
			unlockHighlight()
			blackListScrollFrame.list[i]:LockHighlight()
			removeButton:Enable()
			scrollHighlight = i + scrollOffset
		end)
	end
	blackListScrollFrame.scrollFrame = CreateFrame("ScrollFrame","FauxScrollFrameTestScrollFrame",blackListScrollFrame,"FauxScrollFrameTemplate")
	blackListScrollFrame.scrollFrame:SetPoint("TOPLEFT",blackListScrollFrame,"TOPLEFT",0,-6)
	blackListScrollFrame.scrollFrame:SetPoint("BOTTOMRIGHT",blackListScrollFrame,"BOTTOMRIGHT",-28,6)
	blackListScrollFrame.scrollFrame:SetScript("OnShow",blackListScrollFrame.ScrollFrameUpdate)
	blackListScrollFrame.scrollFrame:SetScript("OnVerticalScroll",function(self,offset)
		FauxScrollFrame_OnVerticalScroll(self,offset,20,blackListScrollFrame.ScrollFrameUpdate)
	end)

	local BLZInterfaceAddon = CreateFrame("Frame")

	BLZInterfaceAddon.name = "EnhancedChatFilter"
	InterfaceOptions_AddCategory(BLZInterfaceAddon)
	BLZInterfaceAddon:Hide()

	BLZInterfaceAddon.button = CreateFrame("Button", "BLZInterfaceAddonButton", BLZInterfaceAddon, "UIPanelButtonTemplate")
	BLZInterfaceAddon.button:SetSize(300,25)
	BLZInterfaceAddon.button:SetText("EnhancedChatFilter")
	BLZInterfaceAddon.button:SetPoint("TOP",BLZInterfaceAddon,"TOP",0,-100)
	BLZInterfaceAddon.button:SetScript("OnClick", function ()
		if InterfaceOptionsFrame:IsShown() then
			InterfaceOptionsFrame:Hide()
		end
		blackListFrame:Show()
		GameMenuButtonContinue:Click()
	end)

	chatLinesLimitSlider:SetPoint("TOPLEFT",tabsFrame,"TOPRIGHT",20,-20)
	chatLinesLimitSlider:SetSize(150,20)
	chatLinesLimitSlider:SetOrientation("HORIZONTAL")
	chatLinesLimitSlider:SetMinMaxValues(1, 100) 
	chatLinesLimitSlider:SetValueStep(1)
	chatLinesLimitSlider:SetObeyStepOnDrag(true)
	chatLinesLimitSlider:SetValue(EnhancedChatFilter:GetchatLinesLimit())
	chatLinesLimitSlider:SetScript("OnValueChanged", function(self, value)
		EnhancedChatFilter:SetchatLinesLimit(value)
		chatLinesLimitValue:SetText(tostring(value))
		chatLinesLimit = value
	end)
	getglobal(chatLinesLimitSlider:GetName() .. 'Low'):SetText('1')
	getglobal(chatLinesLimitSlider:GetName() .. 'High'):SetText('100')
	getglobal(chatLinesLimitSlider:GetName() .. 'Text'):SetText(L["chatLinesLimitSlider"])
	chatLinesLimitSlider.tooltipText = L["chatLinesLimitSliderTooltips"]

	chatLinesLimitValue:SetPoint("TOP",chatLinesLimitSlider,"BOTTOM",0,2)
	chatLinesLimitValue:SetText(tostring(EnhancedChatFilter:GetchatLinesLimit()))

	stringDifferenceLimitSlider:SetPoint("TOPLEFT",chatLinesLimitSlider,"BOTTOMLEFT",0,-40)
	stringDifferenceLimitSlider:SetSize(150,20)
	stringDifferenceLimitSlider:SetOrientation("HORIZONTAL")
	stringDifferenceLimitSlider:SetMinMaxValues(0, 1) 
	stringDifferenceLimitSlider:SetValueStep(0.01)
	stringDifferenceLimitSlider:SetObeyStepOnDrag(true)
	stringDifferenceLimitSlider:SetValue(EnhancedChatFilter:GetstringDifferenceLimit())
	stringDifferenceLimitSlider:SetScript("OnValueChanged", function(self, value)
		EnhancedChatFilter:SetstringDifferenceLimit(value)
		stringDifferenceLimitValue:SetText(string.format("%.2f",tostring(value)))
		stringDifferenceLimit = value
	end)
	getglobal(stringDifferenceLimitSlider:GetName() .. 'Low'):SetText('0')
	getglobal(stringDifferenceLimitSlider:GetName() .. 'High'):SetText('1')
	getglobal(stringDifferenceLimitSlider:GetName() .. 'Text'):SetText(L["stringDifferenceLimitSlider"])
	stringDifferenceLimitSlider.tooltipText = L["stringDifferenceLimitSliderTooltips"]

	stringDifferenceLimitValue:SetPoint("TOP",stringDifferenceLimitSlider,"BOTTOM",0,2)
	stringDifferenceLimitValue:SetText(string.format("%.2f",tostring(EnhancedChatFilter:GetstringDifferenceLimit())))

	tabInitialize()
end

-- create startup event when player logs in
local startUp = CreateFrame("Frame")
startUp:RegisterEvent("PLAYER_LOGIN")

--function that gets called when an event within startUp triggers.
startUp:SetScript("OnEvent", function(self)
	BuildUpFrame()
	print("|cffffff00Enhanced Chat Filter MOD|r|cff3498db v"..ecfVersioin.."|cffecf0f1 - "..L["CommandLine"].." '/ecf'")
end)

function blackListScrollFrame.ScrollFrameUpdate()
	scrollOffset = FauxScrollFrame_GetOffset(blackListScrollFrame.scrollFrame)
	local length=tablelength(EnhancedChatFilter:GetBlackList())
	FauxScrollFrame_Update(blackListScrollFrame.scrollFrame, length, scrollLineLimit, 20)

	unlockHighlight()

	for i=1,scrollLineLimit do
		local idx = scrollOffset+i
		if idx<=length then
			blackListScrollFrame.list[i].name:SetPoint("LEFT",blackListScrollFrame.list[i],"LEFT",2)
			blackListScrollFrame.list[i].name:SetTextColor(1,1,1)
			blackListScrollFrame.list[i].name:SetText(EnhancedChatFilter:GetBlackList()[idx][1])
			if(scrollHighlight == scrollOffset + i) then
				blackListScrollFrame.list[i]:LockHighlight()
			end
			blackListScrollFrame.list[i]:Show()
		else
			blackListScrollFrame.list[i]:Hide()
		end
	end
end

--method run on /ecf
function EnhancedChatFilter:EnhancedChatFilterOpen()
	if(InCombatLockdown()) then return end
	if(blackListFrame:IsVisible() == false) then
		blackListFrame:Show()
	else
		blackListFrame:Hide()
	end
end

--method run on /ecf-clear
function EnhancedChatFilter:EnhancedChatFilterClear()
	EnhancedChatFilter:ClearBlackList()
	blackListScrollFrame.ScrollFrameUpdate()
	print(L["ClearedBlackWordList"])
end

--method run on /ecf-iml
function EnhancedChatFilter:EnhancedChatFilterIgnoreMoreList()
	local ignoreMoreList = EnhancedChatFilter:GetIgnoreMoreList()
	print(L["IgnoreMoreList"]..": ")
	for _,ignorePlayer in ipairs(ignoreMoreList) do
		print(ignorePlayer[1])
	end
end

--method run on /ecf-cleariml
function EnhancedChatFilter:EnhancedChatFilterClearIMList()
	EnhancedChatFilter:ClearIgnoreMoreList()
	print(L["ClearedIgnoreMoreList"])
end

--method run on /ecf-debug
function EnhancedChatFilter:EnhancedChatFilterDebug()
	if(debugMode ~= nil) then
		debugMode=nil
		print("Debug Mode Off!")
	else
		debugMode=true
		print("Debug Mode On!")
	end
end

-----------------------------------------------------------------------
-- ChatFilter 成就刷屏过滤模块
-----------------------------------------------------------------------

local ChatFilter = CreateFrame("Frame")
local achievements, alreadySent = {}, {}
local spamCategories, specialFilters = {[95] = true, [155] = true, [168] = true, [14807] = true, [15165] = true}, {[456] = true, [1400] = true, [1402] = true, [2186] = true, [2187] = true, [2903] = true, [2904] = true, [3004] = true, [3005] = true, [3117] = true, [3259] = true, [3316] = true, [3808] = true, [3809] = true, [3810] = true, [3817] = true, [3818] = true, [3819] = true, [4078] = true, [4079] = true, [4080] = true, [4156] = true, [4576] = true, [4626] = true, [5313] = true, [6954] = true, [7485] = true, [7486] = true, [7487] = true, [8089] = true, [8238] = true, [8246] = true, [8248] = true, [8249] = true, [8260] = true, [8306] = true, [8307] = true, [8398] = true, [8399] = true, [8400] = true, [8401] = true, [8430] = true, [8431] = true, [8432] = true, [8433] = true, [8434] = true, [8435] = true, [8436] = true, [8437] = true, [8438] = true, [8439] = true}

local function SendAchievement(event, achievementID, players)
	if (not players) then return end
	for k in pairs(alreadySent) do alreadySent[k] = nil end
	for i = getn(players), 1, -1 do
		if (alreadySent[players[i].name]) then
			tremove(players, i)
		else
			alreadySent[players[i].name] = true
		end
	end
	if (getn(players) > 1) then
		sort(players, function(a, b) return a.name < b.name end)
	end
	for i = 1, getn(players) do
		local class, color, r, g, b
		if (players[i].guid and players[i].guid:find("Player")) then
			class = select(2, GetPlayerInfoByGUID(players[i].guid))
			color = (CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS)[class]
		end
		if (not color) then
			local info = ChatTypeInfo[strsub(event, 10)]
			r, g, b = info.r, info.g, info.b
		else
			r, g, b = color.r, color.g, color.b
		end
		players[i] = format("|cff%02x%02x%02x|Hplayer:%s|h%s|h|r", r*255, g*255, b*255, players[i].name, players[i].name)
	end
	SendMessage(event, format("[%s]"..L["GotAchievement"].."%s!", table.concat(players, L["And"]), GetAchievementLink(achievementID)))
end

local function achievementReady(id, achievement)
	if (achievement.area and achievement.guild) then
		local playerGuild = GetGuildInfo("player")
		for i = getn(achievement.area), 1, -1 do
			local player = achievement.area[i].name
			if (UnitExists(player) and playerGuild and playerGuild == GetGuildInfo(player)) then
				tinsert(achievement.guild, tremove(achievement.area, i))
			end
		end
	end
	if (achievement.area and getn(achievement.area) > 0) then
		SendAchievement("CHAT_MSG_ACHIEVEMENT", id, achievement.area)
	end
	if (achievement.guild and getn(achievement.guild) > 0) then
		SendAchievement("CHAT_MSG_GUILD_ACHIEVEMENT", id, achievement.guild)
	end
end

local function ChatFrames_OnUpdate(self)
	local found
	for id, achievement in pairs(achievements) do
		if (achievement.timeout <= GetTime()) then
			achievementReady(id, achievement)
			achievements[id] = nil
		end
		found = true
	end
	if (not found) then
		self:SetScript("OnUpdate", nil)
	end
end

local function queueAchievementSpam(event, achievementID, playerdata)
	achievements[achievementID] = achievements[achievementID] or {timeout = GetTime() + 0.5}
	achievements[achievementID][event] = achievements[achievementID][event] or {}
	tinsert(achievements[achievementID][event], playerdata)
	ChatFilter:SetScript("OnUpdate", ChatFrames_OnUpdate)
end

local function ChatFilter_Achievement(self, event, msg, _, _, _, _, _, _, _, _, _, _, guid)
	if EnhancedChatFilter:GetEnableCFA() ~= true then return end
	local achievementID = strmatch(msg, "achievement:(%d+)")
	if (not achievementID) then return end
	achievementID = tonumber(achievementID)
	local Name, Server
	if (guid and guid:find("Player")) then
		Name = select(6, GetPlayerInfoByGUID(guid))
		Server = select(7, GetPlayerInfoByGUID(guid))
		if (Name and Server and strlen(Server) > 0 and Server ~= GetRealmName()) then
			Name = Name.."-"..Server
		end
	end
	local playerdata = {name = Name, guid = guid}
	local categoryID = GetAchievementCategory(achievementID)
	if (spamCategories[categoryID] or spamCategories[select(2, GetCategoryInfo(categoryID))] or specialFilters[achievementID]) then
		queueAchievementSpam((event == "CHAT_MSG_GUILD_ACHIEVEMENT" and "guild" or "area"), achievementID, playerdata)
		return true
	end
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_ACHIEVEMENT", ChatFilter_Achievement)
ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD_ACHIEVEMENT", ChatFilter_Achievement)

--拾取物品过滤器(阿什兰破碎骨骼+暗月马戏团)
local lootBadWords = {118043, 944, 71096}
local function lootfilter(self,_,msg)
	for _, itemId in ipairs(lootBadWords) do
		if (itemId == tonumber(strmatch(msg, "|Hitem:(%d+)"))) then
			return true
		end
	end
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", lootfilter)
